package insalyon.tracecourbe;

import android.content.Context;
import android.util.AttributeSet;
import android.view.View;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;

public class MonPanneau extends  View implements Nextable {

	// Constructeurs et initiateurs
	public MonPanneau(Context context) {
		super(context);
		setBackgroundColor(Color.DKGRAY);
		dessine();
	}
	public MonPanneau(Context context, AttributeSet attrs) {
		super(context, attrs);
		setBackgroundColor(Color.DKGRAY);
		dessine();
	}
	public MonPanneau(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		setBackgroundColor(Color.DKGRAY);
		dessine();
	}
	

  private double xMin;
  private double xMax;
  private double yMin;
  private double yMax;
  private int nbPoints;
    
  
  private Paint functionBrush=new Paint();
  private Paint axisBrush=new Paint();
  private Paint labelsBrush=new Paint();

  public void dessine (){
    dessine(-Math.PI,Math.PI,100);
	axisBrush.setColor(Color.GREEN);
    functionBrush.setColor(Color.WHITE);
    functionBrush.setAntiAlias(true);
	labelsBrush.setColor(Color.RED);
	labelsBrush.setAlpha(255);
	labelsBrush.setAntiAlias(true);
	labelsBrush.setTextSkewX((float)-0.25);	
	labelsBrush.setTextAlign(Paint.Align.CENTER);
  }
  
  public void dessine (double xMin, double xMax, int nbPoints){
    this.xMin=xMin;
    this.xMax=xMax;
    this.nbPoints=nbPoints;
    this.scope=10*(xMax-xMin)/nbPoints;
      
    yMin=f(xMin);
    yMax=f(xMin);
    double x=xMin;
    double delta = (double)(this.xMax - this.xMin) / (double)(this.nbPoints);
    do {
      x +=delta;
      if(f(x)>yMax) yMax=f(x);
      if(f(x)<yMin) yMin=f(x);
    } while (x<=this.xMax);
    
    postInvalidate();
  }


  private double offset=0;
  private double scope=1;
  
  public void next(){
    offset=offset+(scope/10);
    if (this.xMin+offset+scope>this.xMax) offset=0;
    postInvalidate();
  }

  

 /* Méthode donnée dans le sujet */
  protected void onDraw(Canvas g) {
    super.onDraw(g);
    
    if(axes){
      drawAxes(g);
    }
    
    // boucle de tracé point à point
    double x = this.xMin+offset;
    double delta = (double)(this.xMax - this.xMin) / (double)(this.nbPoints);
    int x0 = toScreenX(x); //conversion en coordonnées écran
    int y0 = toScreenY(f(x));
    do {
      x += delta;
      int x1 = toScreenX(x);
      int y1 = toScreenY(f(x));
      // tire un trait entre l'ancien point et le nouveau
      g.drawLine(x0, y0, x1, y1,functionBrush);
      x0 = x1; //près pour le point suivant
      y0 = y1;
    } while (x <= this.xMin+offset+scope);
  }


  /** Pour avoir la meilleure précision possible sur les calculs de toScreen, 
   *  ils sont faits en double précision puis ensuite convertis en int. 
   *  Pour ce qui est des bornes max, width-1 et height-1 correspondent 
   *  respectivement à l'indice d'abscisse et d'ordonnées maximum.
   * */
  private int toScreenX(double x) {
    return (int)((double)((x-xMin)*(getWidth()-1))/(double)((xMax-xMin)));
  }

  private int toScreenY(double y) {
    return getHeight()-1-(int)((double)((y-yMin)*(getHeight()-1))/(double)((yMax-yMin)));
  }
  
  
  
  private boolean axes=true;

  public void setAxes(boolean axes) {
    this.axes = axes;
    postInvalidate();
  }
  
  /** Méthode dessinant des axes sur un graphics pris en paramètre. */
  private void drawAxes(Canvas g){

    // Axe honrizontal
    g.drawLine(toScreenX(xMin), toScreenY(0), toScreenX(xMax), toScreenY(0),axisBrush);
    // Flèche
    g.drawLine(toScreenX(xMax)-5, toScreenY(0)-5, toScreenX(xMax), toScreenY(0),axisBrush);
    g.drawLine(toScreenX(xMax)-5, toScreenY(0)+5, toScreenX(xMax), toScreenY(0),axisBrush);
    
    // Axe vertical
    g.drawLine(toScreenX(0), toScreenY(yMin), toScreenX(0), toScreenY(yMax),axisBrush);
    // Flèche
    g.drawLine(toScreenX(0)-5, toScreenY(yMax)+5, toScreenX(0), toScreenY(yMax),axisBrush);
    g.drawLine(toScreenX(0)+5, toScreenY(yMax)+5, toScreenX(0), toScreenY(yMax),axisBrush);
    


	int h=getHeight();
	labelsBrush.setTextSize(6+(h/30));
	g.drawText(getFunctionsList()[functionNumber],getWidth()/2, getHeight()/2,labelsBrush);
    
  }
  
  
  
  
  
  public String[] getFunctionsList() {
	String[] s={"sin(x²)", "sin²(x)", "cos(x)*sin(x)", "cos(sin(x))"};
	return s;
  }
  
  private int functionNumber=0;
  public void setFunction(int item) {
	  functionNumber=item;
  }
  
  public double f(double x) {
	  
	  switch(functionNumber){
	  	case 0 : return Math.sin(x*x);
	  	case 1 : return Math.sin(x)*Math.sin(x);
	  	case 2 : return Math.cos(x)*Math.sin(x);
	  	case 3 : return Math.cos(Math.sin(x));
	  	default : return 0.0;
	  }
  }
	    
  
}
