package insalyon.tracecourbe;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.TabActivity;
import android.content.DialogInterface;
import android.content.res.Configuration;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.SeekBar;
import android.widget.TabHost;
import android.widget.Toast;
import android.widget.SeekBar.OnSeekBarChangeListener;

public class TraceCourbe extends TabActivity implements OnSeekBarChangeListener {
    /** Called when the activity is first created. */
	
	private GraphView graphView;
	private Button nextGraphBtn ;
	private Button startGraphBtn ;
	private Animateur animGraph=null;
	private boolean isStartedGraph=false;

	
	
	private MonPanneau funcView;
	private Button nextFuncBtn ;
	private Button startFuncBtn ;
	private Button goFuncBtn ;
	private Button axesFuncBtn ;
	private Button fctBtn;
	private Animateur animFunc=null;
	private boolean isStartedFunc=false;
	private EditText xmin;
	private EditText xmax;
	private EditText nbPoints;
	private SeekBar speed;

	
	/** Point de départ de toute activité. 
	 */
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

		initUserInterface();
    }
    

    /** 
     * Pour gérer les rotations. Nécessite aussi la ligne suivante dans le manifest :
     *    android:configChanges="keyboardHidden|orientation"
     *  
     * Pour sauvegarder des données qui ne font pas partie des objets graphiques, 
     * alors on peut suivre le tutoriel suivant : 
     *    http://developer.android.com/resources/articles/faster-screen-orientation-change.html
     */
	@Override
    public void onConfigurationChanged(Configuration newConfig) {
		super.onConfigurationChanged(newConfig);
		setContentView(R.layout.main);
	  
		initUserInterface();
		
        Toast.makeText(getApplicationContext(),"Rotation effectuée", Toast.LENGTH_SHORT).show();
	}
	


	/** Factorisation du code de lien entre le XML et le code Java afin de faciliter la gestion de la rotation du device. 
	 */
    public void initUserInterface() {
    	TabHost mTabHost = getTabHost();
		mTabHost.addTab(mTabHost.newTabSpec("tab_test1").setIndicator("Fonction").setContent(R.id.funcTab));
		mTabHost.addTab(mTabHost.newTabSpec("tab_test0").setIndicator("Graph").setContent(R.id.graphTab));
		mTabHost.setCurrentTab(0);
		
		  
		// Initialisation de la vue 1 : Graph
		
		graphView = (GraphView)findViewById(R.id.graph);
		nextGraphBtn = (Button)findViewById(R.id.nextGraphBtn);
		startGraphBtn = (Button)findViewById(R.id.startGraphBtn);

        graphView.initView(110,50);

		nextGraphBtn.setOnClickListener(new OnClickListener() {
    	    public void onClick(View v) {
    			graphView.next();
        	}
    	});    	
		startGraphBtn.setOnClickListener(new OnClickListener() {
    	    public void onClick(View v) {
    	    	actionBtnGraphStart();
        	}
    	});    	


		
        // Initialisation de la vue 2 : fonction
        
        funcView = (MonPanneau)findViewById(R.id.func);

        nextFuncBtn = (Button)findViewById(R.id.nextFuncBtn);
        startFuncBtn = (Button)findViewById(R.id.startFuncBtn);
		goFuncBtn = (Button)findViewById(R.id.goBtn);
		axesFuncBtn = (Button)findViewById(R.id.axesBtn);
		fctBtn = (Button)findViewById(R.id.fctBtn);
		
        xmin = (EditText)findViewById(R.id.xMin);
		xmax = (EditText)findViewById(R.id.xMax);
		nbPoints = (EditText)findViewById(R.id.nbPoints);

		speed = (SeekBar)findViewById(R.id.speed);
		
		nextFuncBtn.setOnClickListener(new OnClickListener() {
    	    public void onClick(View v) {
    			funcView.next();
        	}
    	});    	
		startFuncBtn.setOnClickListener(new OnClickListener() {
    	    public void onClick(View v) {
    	    	actionBtnFuncStart();
        	}
    	});    	
		
		goFuncBtn.setOnClickListener(new OnClickListener() {
    	    public void onClick(View v) {
    	    	actionBtnFuncGo();
        	}
    	});    	
		
		axesFuncBtn.setOnClickListener(new OnClickListener() {
    	    public void onClick(View v) {
    	    	actionAxes();
        	}
    	});    	
		
		fctBtn.setOnClickListener(new OnClickListener() {
    	    public void onClick(View v) {
    	    	showDialog(0);
        	}
    	});    	
		
		speed.setOnSeekBarChangeListener(this);
    }
	
	
    

    private void actionBtnGraphStart() {
      if(animGraph==null){
        animGraph=new Animateur(graphView,100);
        isStartedGraph=false;
        animGraph.start();
        startGraphBtn.setText("Pause");
      } else {
        if(!isStartedGraph) {
          animGraph.restart();
          startGraphBtn.setText("Pause");
        } else {
          animGraph.pause();
          startGraphBtn.setText("Restart");
        }
      }
      isStartedGraph=!isStartedGraph;
    }


    private void actionBtnFuncStart() {
        if(animFunc==null){
          animFunc=new Animateur(funcView,speed.getProgress());
          isStartedFunc=false;
          animFunc.start();
          startFuncBtn.setText("Pause");
        } else {
          if(!isStartedFunc) {
        	  animFunc.restart();
        	  startFuncBtn.setText("Pause");
          } else {
        	  animFunc.pause();
        	  startFuncBtn.setText("Restart");
          }
        }
        isStartedFunc=!isStartedFunc;
      }
    
    
    private void actionBtnFuncGo() {
        
        
        try {
        	funcView.dessine(Double.parseDouble(xmin.getText().toString()),
                             Double.parseDouble(xmax.getText().toString()),
                             Integer.parseInt(nbPoints.getText().toString()));
        } catch (NumberFormatException ex) {
/*          JOptionPane.showMessageDialog(this,
                                        "Valeur numérique illégale.\nMessage : \n  "+ex.getMessage(),
                                        "Erreur de conversion",
                                        JOptionPane.ERROR_MESSAGE);*/
        }
    }

    
    private boolean lesAxes=true;
    private void actionAxes() {
      lesAxes=!lesAxes;
      if(lesAxes){
    	  axesFuncBtn.setText("Sans axes");
    	  Toast.makeText(getApplicationContext(), "Axes activés", Toast.LENGTH_SHORT).show();
      } else {
    	  axesFuncBtn.setText("Avec axes");
    	  Toast.makeText(getApplicationContext(), "Axes retirés", Toast.LENGTH_SHORT).show();
      }
      funcView.setAxes(lesAxes);
    }





	String[] items; 
	@Override
	protected Dialog onCreateDialog(int id) {
		// TODO Auto-generated method stub
		
		items = funcView.getFunctionsList();
		
		
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle("Choisissez une fonction");
		builder.setItems(items, new DialogInterface.OnClickListener() {
		    public void onClick(DialogInterface dialog, int item) {
		        Toast.makeText(getApplicationContext(), items[item], Toast.LENGTH_SHORT).show();
		        funcView.setFunction(item);
		        actionBtnFuncGo();
		    }
		});
		
		AlertDialog alert = builder.create();
		return alert;
	}
    
    

	@Override
    public void onProgressChanged(SeekBar seekBar, int progress, boolean fromTouch) {}
	
	@Override
	public void onStartTrackingTouch(SeekBar seekBar) {}
	
	@Override
	public void onStopTrackingTouch(SeekBar seekBar) {
    	if(seekBar==speed){
    		if(animFunc!=null){animFunc.setDelay(speed.getProgress());}
	        Toast.makeText(getApplicationContext(),speed.getProgress()+"ms", Toast.LENGTH_SHORT).show();
    	}
	}

}